package no.twofingers.iControl;

import sun.tools.javap.JavaP;
import java.io.*;
import java.util.StringTokenizer;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;

/**
 * This class represents a Java Class that can have DBC assertions turned on or off
 * TODO: Add method visibility invariant
 *
 * @author  <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 */
public class DBCClass extends Setting {

	private static String classpath;
	public static void setClasspath( String cp ) {
		classpath = cp;
	}
	private static String sourceRoot;
	public static void setSourceRoot( String sr ) {
		sourceRoot = sr;
	}

	private final String className;
	private final String qualifiedName;


	// full path file name of the class' source file
	private final String sourceFileName;


	/**
	 * Creates a new DBCMethodImpl
	 *
	 * @param className the name of the class
	 */
	public DBCClass( DBCPackage parent, String className ) {
		super( parent );
		this.className = className;

		// set qualifiedName
		Setting dbcPackage = getParent();
		StringBuffer sb = new StringBuffer();
		if( !dbcPackage.getQualifiedName().equals( DBCPackage.DEFAULT_PACKAGE ) ) {
			sb.append( dbcPackage.getQualifiedName() ).append( "." );
		}
		sb.append( className );
		qualifiedName = sb.toString();

		fireAnalyzingSetting( this );

		// set source file name
		StringBuffer sourceFileNameSb = new StringBuffer( sourceRoot );
		sourceFileNameSb.append( File.separatorChar );
		sourceFileNameSb.append( qualifiedName.replace( '.', File.separatorChar ) );
		sourceFileNameSb.append( ".java" );
		this.sourceFileName = sourceFileNameSb.toString();

		ByteArrayOutputStream javapResult = new ByteArrayOutputStream();
		PrintStream printStream = new PrintStream( javapResult );
		PrintStream systemOut = System.out;
		PrintStream systemErr = System.err;
		System.setOut( printStream );
		System.setErr( new PrintStream( new PipedOutputStream () ) );
		// Braindead JavaP doesn't give exception. We must parse stream in stead
		try {
			JavaP.entry( new String[]{ "-classpath", classpath, qualifiedName } );
		} catch( Throwable t ) {
			System.setOut( systemOut );
			System.setErr( systemErr );
			t.printStackTrace();
		}
		System.setOut( systemOut );
		System.setErr( systemErr );

		BufferedReader javapReader = new BufferedReader( new InputStreamReader( new ByteArrayInputStream( javapResult.toByteArray() ) ) );
		try {
			String line = null;
			StringTokenizer st = null;

			// skip first two lines
			String header = javapReader.readLine();
			if( header == null ) {
				// probably a classpath problem

				// only add the 1st. Don't have sophisticated message box yet.
				if( errorClasses.size() == 0 ) {
					errorClasses.add( qualifiedName );
				}
				// throw new IControlException( "Could not resolve class " + qualifiedName );
			}

			while( ( line = javapReader.readLine() ) != null ) {
				String methodName = null;
				st = new StringTokenizer( line );
				while( st.hasMoreElements() && methodName == null ) {
					String token = st.nextToken();
					int op = token.indexOf( '(' );
					int cp = token.indexOf( ')' );
					if( op != -1 && cp != -1 ) {
						methodName = token.substring( 0, cp + 1 );
						int dot = methodName.indexOf( '.' );
						if( methodName.startsWith( qualifiedName ) ) {
							// only constructors have fully package qualified name. Remove that.
							methodName = methodName.substring( qualifiedName.lastIndexOf( '.' ) + 1, methodName.length() );
						}
					}
				}
				if( methodName != null ) {
					addChild( new DBCMethod( this, methodName ) );
				}
			}

			// Sort the methods. They are not delivered sorted by Javap.
			Collections.sort( getChildren() );
		} catch( IOException e ) {
			// TODO: log4j
			e.printStackTrace();
		}
	}
/*
	protected String getSettings() {
		String result = "";
		if( getSetting( INVARIANT ) == ON ) {
			result = " inv";
		}
		return result;
	}
*/
	protected String getSettings() {
		StringBuffer sb = new StringBuffer( " " );
		boolean comma = false;
		if( getSetting( PRE ) == ON ) {
			sb.append( "pre" );
			comma = true;
		}
		if( getSetting( POST ) == ON ) {
			if( comma ) {
				sb.append( "," );
				comma = true;
			}
			sb.append( "post" );
		}
		if( getSetting( INVARIANT ) == ON ) {
			if( comma ) {
				sb.append( "," );
			}
			sb.append( "inv" );
		}
		return sb.toString();
	}

	public String getDisplayName() {
		return className;
	}

	public String getQualifiedName() {
		return qualifiedName;
	}

	public String getSourceFileName() {
		return sourceFileName;
	}
}

