package no.twofingers.iControl;

/**
 * This interface represents a class method that can have DBC assertions turned on or off
 *
 * @author  <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 *
 * @invariant getSetting( PRE ) == ON || getSetting( PRE ) == OFF
 * @invariant getSetting( POST ) == ON || getSetting( POST ) == OFF
 * @invariant getSetting( INVARIANT ) == ON || getSetting( INVARIANT ) == OFF
 */
public class DBCMethod extends Setting {
	private final String methodName;
	private final String qualifiedName;
	private final boolean _isConstructor;

	/**
	 * Creates a new DBCMethodImpl
	 *
	 * @param parent the enclosing class
	 * @param methodName fully qualified name of the method
	 */
	public DBCMethod( DBCClass parent, String methodName ) {
		super( parent );
		this.methodName = methodName;

		// set qualifiedName
		Setting dbcClass = getParent();
		StringBuffer sb = new StringBuffer().append( dbcClass.getQualifiedName() );
		sb.append( "." ).append( methodName );
		qualifiedName = sb.toString();
		
		// find out whether we're constructor or not.
		String constructorNameStart = new StringBuffer().append( dbcClass.getDisplayName() ).append( "(" ).toString();
		_isConstructor = methodName.startsWith( constructorNameStart );
	}
	
	public boolean isConstructor() {
		return _isConstructor;
	}

	protected String getSettings() {
		StringBuffer sb = new StringBuffer( " " );
		boolean comma = false;
		if( getSetting( PRE ) == ON ) {
			sb.append( "pre" );
			comma = true;
		}
		if( getSetting( POST ) == ON ) {
			if( comma ) {
				sb.append( "," );
			}
			sb.append( "post" );
		}
		return sb.toString();
	}

// Implementation of abstract methods

	public String getSetting( int type ) {
		return settings[ type ];
	}

	public void setSetting( int type, String value ) {
		settings[ type ] = value;
	}

	public String getDisplayName() {
		return methodName;
	}

	public String getQualifiedName() {
		return qualifiedName;
	}
}