/*
 * SettingTreeCellRenderer.java
 *
 * Created on 26. mars 2001, 18:57
 */

package no.twofingers.iControl.swing;

import java.awt.Component;

import javax.swing.JTree;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.tree.DefaultTreeCellRenderer;

import no.twofingers.iControl.DBCProject;
import no.twofingers.iControl.DBCPackage;
import no.twofingers.iControl.DBCClass;
import no.twofingers.iControl.DBCMethod;

/**
 * Renderer for the tree in the tree table
 *
 * @author  <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 */
class SettingTreeCellRenderer extends DefaultTreeCellRenderer {
	
	private final Icon _projectIcon;
	private final Icon _packageIcon;
	private final Icon _classIcon;
	private final Icon _methodIcon;
	private final Icon _constructorIcon;

	/**
	 * Creates a new SettingTreeCellRenderer
	 */
	public SettingTreeCellRenderer() {
		_projectIcon = new ImageIcon( getClass().getResource( "img/project.gif" ) );
		_packageIcon = new ImageIcon( getClass().getResource( "img/package.gif" ) );
		_classIcon = new ImageIcon( getClass().getResource( "img/class.gif" ) );
		_methodIcon = new ImageIcon( getClass().getResource( "img/method.gif" ) );
		_constructorIcon = new ImageIcon( getClass().getResource( "img/constructor.gif" ) );
	}
	
	public Component getTreeCellRendererComponent(
		JTree tree, 
		Object value, 
		boolean sel, 
		boolean expanded, 
		boolean leaf, 
		int row, 
		boolean hasFocus
	) {
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) super.getTreeCellRendererComponent(
			tree, 
			value, 
			sel, 
			expanded, 
			leaf, 
			row, 
			hasFocus
		);
		
		if( value instanceof DBCProject ) {
			renderer.setIcon( _projectIcon );
		} else if( value instanceof DBCPackage ) {
			renderer.setIcon( _packageIcon );
		} else if( value instanceof DBCClass ) {
			renderer.setIcon( _classIcon );
		} else if( value instanceof DBCMethod ) {
			DBCMethod m = (DBCMethod) value;
			if( m.isConstructor() ) {
				renderer.setIcon( _constructorIcon );
			} else {
				renderer.setIcon( _methodIcon );
			}
		}

		return renderer;
	}
}
